@@ -2,7 +2,7 @@
 #
 # The MIT License (MIT)
 #
-# Copyright (c) Tavendo GmbH
+# Copyright (c) Crossbar.io Technologies GmbH
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -27,23 +27,25 @@
 from __future__ import absolute_import
 
 import os
+import mock
 
 if os.environ.get('USE_TWISTED', False):
 
+    from twisted.internet.defer import inlineCallbacks, Deferred, returnValue
+    from twisted.internet.defer import succeed, fail, DeferredList
     from twisted.trial import unittest
-    # import unittest
+    import twisted
+    from six import PY3
 
-    from twisted.internet.defer import inlineCallbacks, Deferred, returnValue, succeed
-    from twisted.python import log
-
-    from autobahn.wamp import message
-    from autobahn.wamp import serializer
-    from autobahn.wamp import role
     from autobahn import util
-    from autobahn.wamp.exception import ApplicationError, NotAuthorized, InvalidUri, ProtocolError
-    from autobahn.wamp import types
-
     from autobahn.twisted.wamp import ApplicationSession
+    from autobahn.wamp import message, role, serializer, types, uri, CloseDetails
+    from autobahn.wamp.request import CallRequest
+    from autobahn.wamp.exception import ApplicationError, NotAuthorized
+    from autobahn.wamp.exception import InvalidUri, ProtocolError
+
+    if PY3:
+        long = int
 
     class MockTransport(object):
 
@@ -64,6 +66,12 @@ if os.environ.get('USE_TWISTED', False):
 
             msg = message.Welcome(self._my_session_id, roles)
             self._handler.onMessage(msg)
+            self._fake_router_session = ApplicationSession()
+
+        def drop_registration(self, reg_id):
+            self._handler.onMessage(
+                message.Unregistered(0, reg_id)
+            )
 
         def send(self, msg):
             if self._log:
@@ -75,9 +83,11 @@ if os.environ.get('USE_TWISTED', False):
             if isinstance(msg, message.Publish):
                 if msg.topic.startswith(u'com.myapp'):
                     if msg.acknowledge:
-                        reply = message.Published(msg.request, util.id())
+                        reply = message.Published(msg.request, self._fake_router_session._request_id_gen.next())
                 elif len(msg.topic) == 0:
                     reply = message.Error(message.Publish.MESSAGE_TYPE, msg.request, u'wamp.error.invalid_uri')
+                elif msg.topic.startswith(u'noreply.'):
+                    pass
                 else:
                     reply = message.Error(message.Publish.MESSAGE_TYPE, msg.request, u'wamp.error.not_authorized')
 
@@ -91,7 +101,9 @@ if os.environ.get('USE_TWISTED', False):
 
                 elif msg.procedure.startswith(u'com.myapp.myproc'):
                     registration = self._registrations[msg.procedure]
-                    request = util.id()
+                    request = self._fake_router_session._request_id_gen.next()
+                    if request in self._invocations:
+                        raise ProtocolError("duplicate invocation")
                     self._invocations[request] = msg.request
                     reply = message.Invocation(
                         request, registration,
@@ -112,7 +124,7 @@ if os.environ.get('USE_TWISTED', False):
                 if topic in self._subscription_topics:
                     reply_id = self._subscription_topics[topic]
                 else:
-                    reply_id = util.id()
+                    reply_id = self._fake_router_session._request_id_gen.next()
                     self._subscription_topics[topic] = reply_id
                 reply = message.Subscribed(msg.request, reply_id)
 
@@ -120,7 +132,7 @@ if os.environ.get('USE_TWISTED', False):
                 reply = message.Unsubscribed(msg.request)
 
             elif isinstance(msg, message.Register):
-                registration = util.id()
+                registration = self._fake_router_session._request_id_gen.next()
                 self._registrations[msg.procedure] = registration
                 reply = message.Registered(msg.request, registration)
 
@@ -163,6 +175,56 @@ if os.environ.get('USE_TWISTED', False):
             # test-case was written
             handler.onClose(False)
 
+        def test_reject_pending(self):
+            handler = ApplicationSession()
+            MockTransport(handler)
+
+            # This could happen if the task waiting on a request gets cancelled
+            deferred = fail(Exception())
+            handler._call_reqs[1] = CallRequest(1, 'foo', deferred, {})
+            handler.onLeave(CloseDetails())
+
+    class TestRegisterDecorator(unittest.TestCase):
+
+        def test_prefix(self):
+
+            class Prefix(ApplicationSession):
+
+                @uri.register(u'method_name')
+                def some_method(self):
+                    pass
+            session = Prefix()
+
+            session._transport = mock.Mock()
+            session.register(session, prefix=u"com.example.prefix.")
+
+            # we should have registered one method, with the prefix
+            # put in front
+            self.assertEqual(1, len(session._transport.mock_calls))
+            call = session._transport.mock_calls[0]
+            self.assertEqual("send", call[0])
+            reg = call.call_list()[0][1][0]
+            self.assertEqual(u"com.example.prefix.method_name", reg.procedure)
+
+        def test_auto_name(self):
+
+            class Magic(ApplicationSession):
+
+                @uri.register(None)
+                def some_method(self):
+                    pass
+            session = Magic()
+
+            session._transport = mock.Mock()
+            session.register(session, prefix=u"com.example.")
+
+            # Should auto-discover name by passing uri=None above
+            self.assertEqual(1, len(session._transport.mock_calls))
+            call = session._transport.mock_calls[0]
+            self.assertEqual("send", call[0])
+            reg = call.call_list()[0][1][0]
+            self.assertEqual(u"com.example.some_method", reg.procedure)
+
     class TestPublisher(unittest.TestCase):
 
         @inlineCallbacks
@@ -185,6 +247,74 @@ if os.environ.get('USE_TWISTED', False):
             publication = yield handler.publish(u'com.myapp.topic1', 1, 2, 3, foo=23, bar='hello', options=types.PublishOptions(exclude_me=False, exclude=[100, 200, 300]))
             self.assertEqual(publication, None)
 
+            publication = yield handler.publish(u'com.myapp.topic1', 1, 2, 3, foo=23, bar='hello', options=types.PublishOptions(exclude_me=False, exclude=[100, 200, 300], retain=True))
+            self.assertEqual(publication, None)
+
+        @inlineCallbacks
+        def test_publish_outstanding_errors(self):
+            handler = ApplicationSession()
+            MockTransport(handler)
+
+            # this publish will "hang" because 'noreply.' URI is
+            # handled specially in MockTransport; so this request will
+            # be outstanding
+            publication = handler.publish(
+                u'noreply.foo',
+                options=types.PublishOptions(acknowledge=True),
+            )
+            # "leave" the session, which should trigger errbacks on
+            # all outstanding requests.
+            details = types.CloseDetails(reason=u'testing', message=u'how are you?')
+            yield handler.onLeave(details)
+
+            # ensure we got our errback
+            try:
+                yield publication
+            except ApplicationError as e:
+                self.assertEqual(u'testing', e.error)
+                self.assertEqual(u'how are you?', e.message)
+
+        @inlineCallbacks
+        def test_publish_outstanding_errors_async_errback(self):
+            handler = ApplicationSession()
+            MockTransport(handler)
+            error_d = Deferred()
+
+            # this publish will "hang" because 'noreply.' URI is
+            # handled specially in MockTransport; so this request will
+            # be outstanding
+            publication_d = handler.publish(
+                u'noreply.foo',
+                options=types.PublishOptions(acknowledge=True),
+            )
+            # further, we add an errback that does some arbitrary async work
+            got_errors = []
+
+            def errback(fail):
+                got_errors.append(fail)
+                return error_d
+            publication_d.addErrback(errback)
+            # "leave" the session, which should trigger errbacks on
+            # all outstanding requests.
+            details = types.CloseDetails(reason=u'testing', message=u'how are you?')
+            handler.onLeave(details)
+
+            # since our errback is async, onLeave should not have
+            # completed yet but we should have already failed the
+            # publication
+            self.assertEqual(1, len(got_errors))
+            # ...now let the async errback continue by completing the
+            # Deferred we returned in our errback (could be fail or
+            # success, shoudln't matter)
+            error_d.callback(None)
+
+            # ensure we (now) get our errback
+            try:
+                yield publication_d
+            except ApplicationError as e:
+                self.assertEqual(u'testing', e.error)
+                self.assertEqual(u'how are you?', e.message)
+
         @inlineCallbacks
         def test_publish_acknowledged(self):
             handler = ApplicationSession()
@@ -277,6 +407,9 @@ if os.environ.get('USE_TWISTED', False):
             subscription = yield handler.subscribe(on_event, u'com.myapp.topic1', options=types.SubscribeOptions(match=u'wildcard'))
             self.assertTrue(type(subscription.id) in (int, long))
 
+            subscription = yield handler.subscribe(on_event, u'com.myapp.topic1', options=types.SubscribeOptions(match=u'wildcard', get_retained=True))
+            self.assertTrue(type(subscription.id) in (int, long))
+
         @inlineCallbacks
         def test_double_subscribe(self):
             handler = ApplicationSession()
@@ -460,37 +593,28 @@ if os.environ.get('USE_TWISTED', False):
             error_instance = RuntimeError("we have a problem")
             got_err_d = Deferred()
 
-            def observer(kw):
-                if kw['isError'] and 'failure' in kw:
-                    fail = kw['failure']
-                    fail.trap(RuntimeError)
-                    if error_instance == fail.value:
-                        got_err_d.callback(True)
-            log.addObserver(observer)
+            def observer(e, msg):
+                if error_instance == e.value:
+                    got_err_d.callback(True)
+            handler.onUserError = observer
 
             def boom():
                 raise error_instance
 
-            try:
-                sub = yield handler.subscribe(boom, u'com.myapp.topic1')
+            sub = yield handler.subscribe(boom, u'com.myapp.topic1')
 
-                # MockTransport gives us the ack reply and then we do our
-                # own event message
-                publish = yield handler.publish(
-                    u'com.myapp.topic1',
-                    options=types.PublishOptions(acknowledge=True, exclude_me=False),
-                )
-                msg = message.Event(sub.id, publish.id)
-                handler.onMessage(msg)
+            # MockTransport gives us the ack reply and then we do our
+            # own event message
+            publish = yield handler.publish(
+                u'com.myapp.topic1',
+                options=types.PublishOptions(acknowledge=True, exclude_me=False),
+            )
+            msg = message.Event(sub.id, publish.id)
+            handler.onMessage(msg)
 
-                # we know it worked if our observer worked and did
-                # .callback on our Deferred above.
-                self.assertTrue(got_err_d.called)
-                # ...otherwise trial will fail the test anyway
-                self.flushLoggedErrors()
-
-            finally:
-                log.removeObserver(observer)
+            # we know it worked if our observer worked and did
+            # .callback on our Deferred above.
+            self.assertTrue(got_err_d.called)
 
         @inlineCallbacks
         def test_unsubscribe(self):
@@ -528,6 +652,58 @@ if os.environ.get('USE_TWISTED', False):
             registration = yield handler.register(on_call, u'com.myapp.procedure1')
             yield registration.unregister()
 
+        def test_unregister_no_such_registration(self):
+            if twisted.version.major < 14:
+                return
+            handler = ApplicationSession()
+            transport = MockTransport(handler)
+
+            with self.assertRaises(ProtocolError) as ctx:
+                transport.send(
+                    message.Unregister(0, 1234)
+                )
+            self.assertIn(
+                "UNREGISTERED received for non-existant registration",
+                str(ctx.exception)
+            )
+
+        @inlineCallbacks
+        def test_unregister_log(self):
+            handler = ApplicationSession()
+            transport = MockTransport(handler)
+
+            def on_call(*args, **kwargs):
+                on_call.called = True
+            on_call.called = False
+
+            registration = yield handler.register(on_call, u'com.myapp.procedure1')
+            transport.drop_registration(registration.id)
+            self.assertFalse(on_call.called)
+
+        def test_on_disconnect_error(self):
+            errors = []
+
+            class AppSess(ApplicationSession):
+                def onUserError(self, e, msg):
+                    errors.append((e.value, msg))
+
+                def onDisconnect(self, foo, bar, quux='snark'):
+                    # this over-ridden onDisconnect takes the wrong args
+                    raise RuntimeError("This shouldn't happen")
+
+            s = AppSess()
+            MockTransport(s)
+
+            # when the transport closes, it calls onClose which should
+            # dispatch onDisconnect
+            s.onClose(False)
+
+            # ...but we should collect an error, because our
+            # onDisconnect takes the wrong arguments.
+            self.assertEqual(len(errors), 1)
+            self.assertTrue(isinstance(errors[0][0], TypeError))
+
+    class TestInvoker(unittest.TestCase):
         @inlineCallbacks
         def test_invoke(self):
             handler = ApplicationSession()
@@ -541,11 +717,65 @@ if os.environ.get('USE_TWISTED', False):
             res = yield handler.call(u'com.myapp.myproc1')
             self.assertEqual(res, 23)
 
+        @inlineCallbacks
+        def test_invoke_twice(self):
+            handler = ApplicationSession()
+            MockTransport(handler)
+
+            def myproc1():
+                return 23
+
+            yield handler.register(myproc1, u'com.myapp.myproc1')
+
+            d0 = handler.call(u'com.myapp.myproc1')
+            d1 = handler.call(u'com.myapp.myproc1')
+            res = yield DeferredList([d0, d1])
+            self.assertEqual(res, [(True, 23), (True, 23)])
+
+        @inlineCallbacks
+        def test_invoke_request_id_sequences(self):
+            """
+            make sure each session independently generates sequential IDs
+            """
+            handler0 = ApplicationSession()
+            handler1 = ApplicationSession()
+            trans0 = MockTransport(handler0)
+            trans1 = MockTransport(handler1)
+
+            # the ID sequences for each session should both start at 0
+            # (the register) and then increment for the call()
+            def verify_seq_id(orig, msg):
+                if isinstance(msg, message.Register):
+                    self.assertEqual(msg.request, 1)
+                elif isinstance(msg, message.Call):
+                    self.assertEqual(msg.request, 2)
+                return orig(msg)
+            orig0 = trans0.send
+            orig1 = trans1.send
+            trans0.send = lambda msg: verify_seq_id(orig0, msg)
+            trans1.send = lambda msg: verify_seq_id(orig1, msg)
+
+            def myproc1():
+                return 23
+
+            yield handler0.register(myproc1, u'com.myapp.myproc1')
+            yield handler1.register(myproc1, u'com.myapp.myproc1')
+
+            d0 = handler0.call(u'com.myapp.myproc1')
+            d1 = handler1.call(u'com.myapp.myproc1')
+            res = yield DeferredList([d0, d1])
+            self.assertEqual(res, [(True, 23), (True, 23)])
+
         @inlineCallbacks
         def test_invoke_user_raises(self):
             handler = ApplicationSession()
             handler.traceback_app = True
             MockTransport(handler)
+            errors = []
+
+            def log_error(e, msg):
+                errors.append((e.value, msg))
+            handler.onUserError = log_error
 
             name_error = NameError('foo')
 
@@ -566,9 +796,8 @@ if os.environ.get('USE_TWISTED', False):
 
             # also, we should have logged the real NameError to
             # Twisted.
-            errs = self.flushLoggedErrors()
-            self.assertEqual(1, len(errs))
-            self.assertEqual(name_error, errs[0].value)
+            self.assertEqual(1, len(errors))
+            self.assertEqual(name_error, errors[0][0])
 
         @inlineCallbacks
         def test_invoke_progressive_result(self):
@@ -584,7 +813,7 @@ if os.environ.get('USE_TWISTED', False):
                     yield succeed(i)
                 returnValue(42)
 
-            progressive = map(lambda _: Deferred(), range(10))
+            progressive = list(map(lambda _: Deferred(), range(10)))
 
             def progress(arg):
                 progressive[arg].callback(arg)
@@ -622,6 +851,11 @@ if os.environ.get('USE_TWISTED', False):
 
             got_progress = Deferred()
             progress_error = NameError('foo')
+            logged_errors = []
+
+            def got_error(e, msg):
+                logged_errors.append((e.value, msg))
+            handler.onUserError = got_error
 
             def progress(arg, something=None):
                 self.assertEqual('nothing', something)
@@ -641,15 +875,15 @@ if os.environ.get('USE_TWISTED', False):
                 options=types.CallOptions(on_progress=progress),
                 key='word',
             )
+
             self.assertEqual(42, res)
             # our progress handler raised an error, but not before
             # recording success.
             self.assertTrue(got_progress.called)
             self.assertEqual('life', got_progress.result)
             # make sure our progress-handler error was logged
-            errs = self.flushLoggedErrors()
-            self.assertEqual(1, len(errs))
-            self.assertEqual(progress_error, errs[0].value)
+            self.assertEqual(1, len(logged_errors))
+            self.assertEqual(progress_error, logged_errors[0][0])
 
         @inlineCallbacks
         def test_invoke_progressive_result_no_args(self):
@@ -716,6 +950,43 @@ if os.environ.get('USE_TWISTED', False):
             self.assertTrue(got_progress.called)
             self.assertEqual('word', got_progress.result)
 
+        @inlineCallbacks
+        def test_call_exception_runtimeerror(self):
+            handler = ApplicationSession()
+            MockTransport(handler)
+            exception = RuntimeError("a simple error")
+
+            def raiser():
+                raise exception
+
+            registration0 = yield handler.register(raiser, u"com.myapp.myproc_error")
+            try:
+                yield handler.call(u'com.myapp.myproc_error')
+                self.fail()
+            except Exception as e:
+                self.assertIsInstance(e, ApplicationError)
+                self.assertEqual(e.error_message(), "wamp.error.runtime_error: a simple error")
+            finally:
+                yield registration0.unregister()
+
+        @inlineCallbacks
+        def test_call_exception_bare(self):
+            handler = ApplicationSession()
+            MockTransport(handler)
+            exception = Exception()
+
+            def raiser():
+                raise exception
+
+            registration0 = yield handler.register(raiser, u"com.myapp.myproc_error")
+            try:
+                yield handler.call(u'com.myapp.myproc_error')
+                self.fail()
+            except Exception as e:
+                self.assertIsInstance(e, ApplicationError)
+            finally:
+                yield registration0.unregister()
+
         # ## variant 1: works
         # def test_publish1(self):
         #    d = self.handler.publish(u'de.myapp.topic1')
@@ -731,7 +1002,3 @@ if os.environ.get('USE_TWISTED', False):
         # def test_publish3(self):
         #    with self.assertRaises(ApplicationError):
         #       yield self.handler.publish(u'de.myapp.topic1')
-
-
-if __name__ == '__main__':
-    unittest.main()
