@@ -4,20 +4,20 @@ from autobahn.asyncio.wamp import ApplicationSession, ApplicationRunner
 
 
 class MyComponent(ApplicationSession):
-    @asyncio.coroutine
-    def onJoin(self, details):
+    async def onJoin(self, details):
         # a remote procedure; see frontend.py for a Python front-end
         # that calls this. Any language with WAMP bindings can now call
         # this procedure if its connected to the same router and realm.
         def add2(x, y):
             return x + y
-        yield from self.register(add2, u'com.myapp.add2')
+        reg = await self.register(add2, u'com.myapp.add2')
+        print("registered 'com.myapp.add2' with id {}".format(reg.id))
 
         # publish an event every second. The event payloads can be
         # anything JSON- and msgpack- serializable
         while True:
             self.publish(u'com.myapp.hello', 'Hello, world!')
-            yield from asyncio.sleep(1)
+            await asyncio.sleep(1)
 
 
 if __name__ == '__main__':
