@@ -2,7 +2,7 @@
 #
 # The MIT License (MIT)
 #
-# Copyright (c) Tavendo GmbH
+# Copyright (c) Crossbar.io Technologies GmbH
 #
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to deal
@@ -58,8 +58,9 @@ if __name__ == '__main__':
     # we server static files under "/" ..
     root = File(".")
 
-    # and our WebSocket server under "/ws"
-    root.putChild(u"ws", resource)
+    # and our WebSocket server under "/ws" (note that Twisted uses
+    # bytes for URIs)
+    root.putChild(b"ws", resource)
 
     # both under one Twisted Web Site
     site = Site(root)
